<?php

namespace Drupal\performance_budget\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Queue\RequeueException;
use Drupal\performance_budget\Event\AggregateJobCompleteEvent;
use Drupal\web_page_archive\Entity\WebPageArchiveInterface;

/**
 * Class AggregateBatchController.
 */
class AggregateBatchController extends ControllerBase {

  /**
   * Retrieves a queue for the specified WPA entity.
   *
   * @param \Drupal\web_page_archive\Entity\WebPageArchiveInterface $wpa
   *   The web page archive config entity.
   * @param bool $empty
   *   Indicates whether or not we should purge the queue.
   */
  public static function getQueue(WebPageArchiveInterface $wpa, $empty = FALSE) {
    $queue_name = static::getQueueName($wpa);
    $queue = \Drupal::service('queue')->get($queue_name);
    if ($empty) {
      $queue->deleteQueue();
      $queue->createQueue();
      \Drupal::state()->set($queue_name, []);
    }
    return $queue;
  }

  /**
   * Retrieve the name of a queue for the specified WPA entity.
   *
   * @param \Drupal\web_page_archive\Entity\WebPageArchiveInterface $wpa
   *   The web page archive config entity.
   *
   * @return string
   *   Name of the queue.
   */
  public static function getQueueName(WebPageArchiveInterface $wpa) {
    $run = $wpa->getRunEntity();
    return "pb_aggregate_runs:{$run->uuid()}";
  }

  /**
   * Enqueues the job run data for the specified config entity.
   *
   * @param \Drupal\web_page_archive\Entity\WebPageArchiveInterface $wpa
   *   The web page archive config entity.
   */
  public static function enqueueJobRunData(WebPageArchiveInterface $wpa) {
    $run_storage = \Drupal::entityTypeManager()->getStorage('web_page_archive_run');

    // Get a brand new queue for this particular run.
    $queue_name = static::getQueueName($wpa);
    $queue = static::getQueue($wpa, TRUE);

    $run = $wpa->getRunEntity();
    foreach ($run_storage->revisionIds($run) as $vid) {
      $item = [
        'queue_name' => $queue_name,
        'vid' => $vid,
      ];
      $queue->createItem($item);
    }
  }

  /**
   * Adds up to $items_to_process number of items from the queue to a batch.
   *
   * If $items_to_process < 0 attempt to add entire queue to batch.
   *
   * @param \Drupal\web_page_archive\Entity\WebPageArchiveInterface $web_page_archive
   *   The web page archive config entity.
   * @param int $items_to_process
   *   The number of items to process.
   */
  public static function setBatch(WebPageArchiveInterface $web_page_archive, $items_to_process = -1) {
    $queue = static::getQueue($web_page_archive);
    $queue_worker = \Drupal::service('plugin.manager.queue_worker')->createInstance('pb_aggregate_runs_queue_worker');

    // Create capture job batch.
    $batch = [
      'title' => \t('Process all web page test data with batch'),
      'operations' => [],
      'finished' => [static::class, 'batchFinished'],
    ];

    // If negative, or if count is too high, set count to queue size.
    if ($items_to_process < 0 || $items_to_process > $queue->numberOfItems()) {
      $items_to_process = $queue->numberOfItems();
    }

    // Create batch operations.
    for ($i = 0; $i < $items_to_process; $i++) {
      $batch['operations'][] = [
        [static::class, 'batchProcess'],
        [$web_page_archive],
      ];
    }

    // Adds the batch sets.
    batch_set($batch);
  }

  /**
   * Common batch processing callback for all operations.
   *
   * @param \Drupal\web_page_archive\Entity\WebPageArchiveInterface $web_page_archive
   *   The web page archive config entity.
   * @param array $context
   *   Contains context for the current batch job.
   */
  public static function batchProcess(WebPageArchiveInterface $web_page_archive, array &$context = NULL) {
    if (empty($context['results']['entity'])) {
      $context['results']['entity'] = $web_page_archive;
    }

    $queue_worker = \Drupal::service('plugin.manager.queue_worker')->createInstance('pb_aggregate_runs_queue_worker');
    $queue = static::getQueue($web_page_archive);

    if ($item = $queue->claimItem()) {
      try {
        $processed = $queue_worker->processItem($item->data);
        if (!isset($processed)) {
          throw new RequeueException(t('Still Running'));
        }
        $queue->deleteItem($item);
        return TRUE;
      }
      catch (RequeueException $e) {
        $queue->releaseItem($item);
      }
      catch (SuspendQueueException $e) {
        $queue->releaseItem($item);
        watchdog_exception($e);
      }
      catch (\Exception $e) {
        // In case of any other kind of exception, log it and remove it from
        // the queue to prevent queues from getting stuck.
        $queue->deleteItem($item);
        watchdog_exception('performance_budget', $e);
      }

      return FALSE;
    }
  }

  /**
   * Batch finished callback.
   *
   * @param bool $success
   *   Indicates whether or not the job succeeeded.
   * @param array $results
   *   Contains results from the batch job.
   * @param array $operations
   *   List of operations run on the batch.
   */
  public static function batchFinished($success, array $results, array $operations) {
    if ($success) {
      // Dispatch an event.
      if (isset($results['entity'])) {
        $event = new AggregateJobCompleteEvent($results['entity']);
        $event_dispatcher = \Drupal::service('event_dispatcher');
        $event_dispatcher->dispatch($event::EVENT_NAME, $event);
      }
    }
    else {
      $error_operation = reset($operations);
      $values = [
        '@operation' => $error_operation[0],
        '@args' => print_r($error_operation[0], TRUE),
      ];
      \Drupal::messenger()->addError(t('An error occurred while processing @operation with arguments : @args', $values));
    }
  }

}
